==> agentic_fleet-0.4.81.tar.gz: agentic_fleet-0.4.81/agentic_fleet/core/tools/mind_map/mind_map_tool.py <==
"""
Mind Map Tool Module.

This module provides tools for constructing and analyzing knowledge graphs
to track logical relationships in complex reasoning chains.
"""

from typing import Any, Dict, List, Optional, Set, Tuple

import networkx as nx
from pydantic import BaseModel


class Entity(BaseModel):
    """Represents a node in the knowledge graph."""
    id: str
    label: str
    type: str
    attributes: Dict[str, Any] = {}
    confidence: float = 1.0


class Relationship(BaseModel):
    """Represents an edge in the knowledge graph."""
    source: str
    target: str
    type: str
    attributes: Dict[str, Any] = {}
    weight: float = 1.0


class MindMapTool:
    """
    Tool for constructing and analyzing knowledge graphs to track
    logical relationships and dependencies in reasoning chains.
    """

    def __init__(
        self,
        graph_type: str = "directed",
        max_nodes: int = 50,
        clustering_threshold: float = 0.7
    ) -> None:
        """
        Initialize the Mind Map Tool.

        Args:
            graph_type: Type of graph to construct (directed/undirected)
            max_nodes: Maximum number of nodes in the mind map
            clustering_threshold: Threshold for community clustering
        """
        self.graph = nx.DiGraph() if graph_type == "directed" else nx.Graph()
        self.max_nodes = max_nodes
        self.clustering_threshold = clustering_threshold
        self.entity_types: Set[str] = set()
        self.relationship_types: Set[str] = set()

    def add_entities(self, entities_data: str) -> None:
        """
        Add entities to the knowledge graph from structured data.

        Args:
            entities_data: Structured string containing entity information
        """
        # Parse entities from the structured data
        # This is a placeholder - actual implementation would parse the LLM output
        entities = self._parse_entities(entities_data)
        
        # Add entities to the graph
        for entity in entities:
            if len(self.graph.nodes) >= self.max_nodes:
                break
            
            self.graph.add_node(
                entity.id,
                label=entity.label,
                type=entity.type,
                attributes=entity.attributes,
                confidence=entity.confidence
            )
            self.entity_types.add(entity.type)

    def add_relationships(self, relationships_data: str) -> None:
        """
        Add relationships between entities in the knowledge graph.

        Args:
            relationships_data: Structured string containing relationship information
        """
        # Parse relationships from the structured data
        relationships = self._parse_relationships(relationships_data)
        
        # Add relationships to the graph
        for rel in relationships:
            if rel.source in self.graph.nodes and rel.target in self.graph.nodes:
                self.graph.add_edge(
                    rel.source,
                    rel.target,
                    type=rel.type,
                    attributes=rel.attributes,
                    weight=rel.weight
                )
                self.relationship_types.add(rel.type)

    def analyze_graph(self) -> str:
        """
        Analyze the knowledge graph to extract insights.

        Returns:
            String containing analysis results
        """
        analysis = []
        
        # Basic graph metrics
        analysis.append(f"Nodes: {len(self.graph.nodes)}")
        analysis.append(f"Edges: {len(self.graph.edges)}")
        
        # Identify central nodes
        centrality = nx.degree_centrality(self.graph)
        central_nodes = sorted(
            centrality.items(),
            key=lambda x: x[1],
            reverse=True
        )[:5]
        
        analysis.append("\nKey Concepts:")
        for node, score in central_nodes:
            node_data = self.graph.nodes[node]
            analysis.append(
                f"- {node_data['label']} "
                f"(Centrality: {score:.2f}, "
                f"Type: {node_data['type']})"
            )
        
        # Identify communities
        communities = list(nx.community.greedy_modularity_communities(
            self.graph.to_undirected()
        ))
        
        analysis.append("\nThematic Clusters:")
        for i, community in enumerate(communities):
            nodes = [self.graph.nodes[n]["label"] for n in community]
            analysis.append(f"Cluster {i+1}: {', '.join(nodes)}")
        
        # Analyze paths and dependencies
        analysis.append("\nKey Dependencies:")
        try:
            for path in nx.all_simple_paths(
                self.graph,
                source=central_nodes[0][0],
                target=central_nodes[-1][0],
                cutoff=4
            ):
                path_labels = [
                    self.graph.nodes[n]["label"]
                    for n in path
                ]
                analysis.append(f"- {' → '.join(path_labels)}")
        except (nx.NetworkXNoPath, IndexError):
            pass
        
        return "\n".join(analysis)

    def get_graph_state(self) -> Dict[str, Any]:
        """
        Get the current state of the knowledge graph.

        Returns:
            Dictionary containing the graph state
        """
        return {
            "nodes": [
                {
                    "id": node,
                    "label": data["label"],
                    "type": data["type"],
                    "attributes": data["attributes"],
                    "confidence": data.get("confidence", 1.0)
                }
                for node, data in self.graph.nodes(data=True)
            ],
            "edges": [
                {
                    "source": source,
                    "target": target,
                    "type": data["type"],
                    "attributes": data["attributes"],
                    "weight": data.get("weight", 1.0)
                }
                for source, target, data in self.graph.edges(data=True)
            ],
            "metadata": {
                "entity_types": list(self.entity_types),
                "relationship_types": list(self.relationship_types),
                "node_count": len(self.graph.nodes),
                "edge_count": len(self.graph.edges)
            }
        }

    def clear_graph(self) -> None:
        """Clear the current knowledge graph."""
        self.graph.clear()
        self.entity_types.clear()
        self.relationship_types.clear()

    def _parse_entities(self, entities_data: str) -> List[Entity]:
        """
        Parse entities from structured data.
        
        Args:
            entities_data: Structured string containing entity information
            
        Returns:
            List of Entity objects
        """
        # This is a placeholder - actual implementation would parse LLM output
        # into structured Entity objects
        entities = []
        # Add parsing logic here
        return entities

    def _parse_relationships(self, relationships_data: str) -> List[Relationship]:
        """
        Parse relationships from structured data.
        
        Args:
            relationships_data: Structured string containing relationship information
            
        Returns:
            List of Relationship objects
        """
        # This is a placeholder - actual implementation would parse LLM output
        # into structured Relationship objects
        relationships = []
        # Add parsing logic here
        return relationships

==> agentic_fleet-0.4.90.tar.gz: agentic_fleet-0.4.90/agentic_fleet/core/tools/mind_map/mind_map_tool.py <==
"""
Mind Map Tool Module.

This module provides tools for constructing and analyzing knowledge graphs
to track logical relationships in complex reasoning chains.
"""

from typing import Any, Dict, List, Set

import networkx as nx
from pydantic import BaseModel


class Entity(BaseModel):
    """Represents a node in the knowledge graph."""

    id: str
    label: str
    type: str
    attributes: Dict[str, Any] = {}
    confidence: float = 1.0


class Relationship(BaseModel):
    """Represents an edge in the knowledge graph."""

    source: str
    target: str
    type: str
    attributes: Dict[str, Any] = {}
    weight: float = 1.0


class MindMapTool:
    """
    Tool for constructing and analyzing knowledge graphs to track
    logical relationships and dependencies in reasoning chains.
    """

    def __init__(
        self, graph_type: str = "directed", max_nodes: int = 50, clustering_threshold: float = 0.7
    ) -> None:
        """
        Initialize the Mind Map Tool.

        Args:
            graph_type: Type of graph to construct (directed/undirected)
            max_nodes: Maximum number of nodes in the mind map
            clustering_threshold: Threshold for community clustering
        """
        self.graph = nx.DiGraph() if graph_type == "directed" else nx.Graph()
        self.max_nodes = max_nodes
        self.clustering_threshold = clustering_threshold
        self.entity_types: Set[str] = set()
        self.relationship_types: Set[str] = set()

    def add_entities(self, entities_data: str) -> None:
        """
        Add entities to the knowledge graph from structured data.

        Args:
            entities_data: Structured string containing entity information
        """
        # Parse entities from the structured data
        # This is a placeholder - actual implementation would parse the LLM output
        entities = self._parse_entities(entities_data)

        # Add entities to the graph
        for entity in entities:
            if len(self.graph.nodes) >= self.max_nodes:
                break

            self.graph.add_node(
                entity.id,
                label=entity.label,
                type=entity.type,
                attributes=entity.attributes,
                confidence=entity.confidence,
            )
            self.entity_types.add(entity.type)

    def add_relationships(self, relationships_data: str) -> None:
        """
        Add relationships between entities in the knowledge graph.

        Args:
            relationships_data: Structured string containing relationship information
        """
        # Parse relationships from the structured data
        relationships = self._parse_relationships(relationships_data)

        # Add relationships to the graph
        for rel in relationships:
            if rel.source in self.graph.nodes and rel.target in self.graph.nodes:
                self.graph.add_edge(
                    rel.source,
                    rel.target,
                    type=rel.type,
                    attributes=rel.attributes,
                    weight=rel.weight,
                )
                self.relationship_types.add(rel.type)

    def analyze_graph(self) -> str:
        """
        Analyze the knowledge graph to extract insights.

        Returns:
            String containing analysis results
        """
        analysis = []

        # Basic graph metrics
        analysis.append(f"Nodes: {len(self.graph.nodes)}")
        analysis.append(f"Edges: {len(self.graph.edges)}")

        # Identify central nodes
        centrality = nx.degree_centrality(self.graph)
        central_nodes = sorted(centrality.items(), key=lambda x: x[1], reverse=True)[:5]

        analysis.append("\nKey Concepts:")
        for node, score in central_nodes:
            node_data = self.graph.nodes[node]
            analysis.append(
                f"- {node_data['label']} "
                f"(Centrality: {score:.2f}, "
                f"Type: {node_data['type']})"
            )

        # Identify communities
        communities = list(nx.community.greedy_modularity_communities(self.graph.to_undirected()))

        analysis.append("\nThematic Clusters:")
        for i, community in enumerate(communities):
            nodes = [self.graph.nodes[n]["label"] for n in community]
            analysis.append(f"Cluster {i+1}: {', '.join(nodes)}")

        # Analyze paths and dependencies
        analysis.append("\nKey Dependencies:")
        try:
            for path in nx.all_simple_paths(
                self.graph, source=central_nodes[0][0], target=central_nodes[-1][0], cutoff=4
            ):
                path_labels = [self.graph.nodes[n]["label"] for n in path]
                analysis.append(f"- {' → '.join(path_labels)}")
        except (nx.NetworkXNoPath, IndexError):
            pass

        return "\n".join(analysis)

    def get_graph_state(self) -> Dict[str, Any]:
        """
        Get the current state of the knowledge graph.

        Returns:
            Dictionary containing the graph state
        """
        return {
            "nodes": [
                {
                    "id": node,
                    "label": data["label"],
                    "type": data["type"],
                    "attributes": data["attributes"],
                    "confidence": data.get("confidence", 1.0),
                }
                for node, data in self.graph.nodes(data=True)
            ],
            "edges": [
                {
                    "source": source,
                    "target": target,
                    "type": data["type"],
                    "attributes": data["attributes"],
                    "weight": data.get("weight", 1.0),
                }
                for source, target, data in self.graph.edges(data=True)
            ],
            "metadata": {
                "entity_types": list(self.entity_types),
                "relationship_types": list(self.relationship_types),
                "node_count": len(self.graph.nodes),
                "edge_count": len(self.graph.edges),
            },
        }

    def clear_graph(self) -> None:
        """Clear the current knowledge graph."""
        self.graph.clear()
        self.entity_types.clear()
        self.relationship_types.clear()

    def _parse_entities(self, entities_data: str) -> List[Entity]:
        """
        Parse entities from structured data.

        Args:
            entities_data: Structured string containing entity information

        Returns:
            List of Entity objects
        """
        # This is a placeholder - actual implementation would parse LLM output
        # into structured Entity objects
        entities = []
        # Add parsing logic here
        return entities

    def _parse_relationships(self, relationships_data: str) -> List[Relationship]:
        """
        Parse relationships from structured data.

        Args:
            relationships_data: Structured string containing relationship information

        Returns:
            List of Relationship objects
        """
        # This is a placeholder - actual implementation would parse LLM output
        # into structured Relationship objects
        relationships = []
        # Add parsing logic here
        return relationships
